@@ -8,9 +8,13 @@ import (
 )
 
 func (n *Naive) configureHTTP3Listener() error {
+	tlsConfig, err := n.tlsConfig.Config()
+	if err != nil {
+		return err
+	}
 	h3Server := &http3.Server{
 		Port:      int(n.listenOptions.ListenPort),
-		TLSConfig: n.tlsConfig.Config(),
+		TLSConfig: tlsConfig,
 		Handler:   n,
 	}
 
