@@ -3,15 +3,14 @@ package inbound
 import (
 	"context"
 	"net"
-	"net/http"
 	"os"
 
 	"github.com/sagernet/sing-box/adapter"
-	"github.com/sagernet/sing-box/common/pipelistener"
-	"github.com/sagernet/sing-box/common/trafficcontrol"
 	C "github.com/sagernet/sing-box/constant"
 	"github.com/sagernet/sing-box/log"
 	"github.com/sagernet/sing-box/option"
+	"github.com/sagernet/sing-shadowsocks"
+	"github.com/sagernet/sing-shadowsocks/shadowaead"
 	"github.com/sagernet/sing-shadowsocks/shadowaead_2022"
 	"github.com/sagernet/sing/common"
 	"github.com/sagernet/sing/common/auth"
@@ -28,12 +27,8 @@ var (
 
 type ShadowsocksMulti struct {
 	myInboundAdapter
-	service        *shadowaead_2022.MultiService[int]
-	users          []option.ShadowsocksUser
-	controlEnabled bool
-	controller     *http.Server
-	controllerPipe *pipelistener.Listener
-	trafficManager *trafficcontrol.Manager[int]
+	service shadowsocks.MultiService[int]
+	users   []option.ShadowsocksUser
 }
 
 func newShadowsocksMulti(ctx context.Context, router adapter.Router, logger log.ContextLogger, tag string, options option.ShadowsocksInboundOptions) (*ShadowsocksMulti, error) {
@@ -56,26 +51,30 @@ func newShadowsocksMulti(ctx context.Context, router adapter.Router, logger log.
 	} else {
 		udpTimeout = int64(C.UDPTimeout.Seconds())
 	}
-	service, err := shadowaead_2022.NewMultiServiceWithPassword[int](
-		options.Method,
-		options.Password,
-		udpTimeout,
-		adapter.NewUpstreamContextHandler(inbound.newConnection, inbound.newPacketConnection, inbound),
+	var (
+		service shadowsocks.MultiService[int]
+		err     error
 	)
-	users := options.Users
-	if options.ControlPassword != "" {
-		inbound.controlEnabled = true
-		users = append([]option.ShadowsocksUser{{
-			Name:     "control",
-			Password: options.ControlPassword,
-		}}, users...)
-		inbound.controller = &http.Server{Handler: inbound.createHandler()}
-		inbound.trafficManager = trafficcontrol.NewManager[int]()
+	if common.Contains(shadowaead_2022.List, options.Method) {
+		service, err = shadowaead_2022.NewMultiServiceWithPassword[int](
+			options.Method,
+			options.Password,
+			udpTimeout,
+			adapter.NewUpstreamContextHandler(inbound.newConnection, inbound.newPacketConnection, inbound),
+			router.TimeFunc(),
+		)
+	} else if common.Contains(shadowaead.List, options.Method) {
+		service, err = shadowaead.NewMultiService[int](
+			options.Method,
+			udpTimeout,
+			adapter.NewUpstreamContextHandler(inbound.newConnection, inbound.newPacketConnection, inbound))
+	} else {
+		return nil, E.New("unsupported method: " + options.Method)
 	}
 	if err != nil {
 		return nil, err
 	}
-	err = service.UpdateUsersWithPasswords(common.MapIndexed(users, func(index int, user option.ShadowsocksUser) int {
+	err = service.UpdateUsersWithPasswords(common.MapIndexed(options.Users, func(index int, user option.ShadowsocksUser) int {
 		return index
 	}), common.Map(options.Users, func(user option.ShadowsocksUser) string {
 		return user.Password
@@ -85,30 +84,10 @@ func newShadowsocksMulti(ctx context.Context, router adapter.Router, logger log.
 	}
 	inbound.service = service
 	inbound.packetUpstream = service
-	inbound.users = users
+	inbound.users = options.Users
 	return inbound, err
 }
 
-func (h *ShadowsocksMulti) Start() error {
-	if h.controlEnabled {
-		h.controllerPipe = pipelistener.New(16)
-		go func() {
-			err := h.controller.Serve(h.controllerPipe)
-			if err != nil {
-				h.newError(E.Cause(err, "controller serve error"))
-			}
-		}()
-	}
-	return h.myInboundAdapter.Start()
-}
-
-func (h *ShadowsocksMulti) Close() error {
-	if h.controlEnabled {
-		h.controllerPipe.Close()
-	}
-	return h.myInboundAdapter.Close()
-}
-
 func (h *ShadowsocksMulti) NewConnection(ctx context.Context, conn net.Conn, metadata adapter.InboundContext) error {
 	return h.service.NewConnection(adapter.WithContext(log.ContextWithNewID(ctx), &metadata), conn, adapter.UpstreamMetadata(metadata))
 }
@@ -126,11 +105,6 @@ func (h *ShadowsocksMulti) newConnection(ctx context.Context, conn net.Conn, met
 	if !loaded {
 		return os.ErrInvalid
 	}
-	if userIndex == 0 && h.controlEnabled {
-		h.logger.InfoContext(ctx, "inbound control connection")
-		h.controllerPipe.Serve(conn)
-		return nil
-	}
 	user := h.users[userIndex].Name
 	if user == "" {
 		user = F.ToString(userIndex)
