@@ -1,7 +1,9 @@
 package outbound
 
 import (
+	"bytes"
 	"context"
+	"encoding/base64"
 	"math/rand"
 	"net"
 	"os"
@@ -21,7 +23,10 @@ import (
 	"golang.org/x/crypto/ssh"
 )
 
-var _ adapter.Outbound = (*SSH)(nil)
+var (
+	_ adapter.Outbound                = (*SSH)(nil)
+	_ adapter.InterfaceUpdateListener = (*SSH)(nil)
+)
 
 type SSH struct {
 	myOutboundAdapter
@@ -29,6 +34,7 @@ type SSH struct {
 	dialer            N.Dialer
 	serverAddr        M.Socksaddr
 	user              string
+	hostKey           []ssh.PublicKey
 	hostKeyAlgorithms []string
 	clientVersion     string
 	authMethod        []ssh.AuthMethod
@@ -40,11 +46,12 @@ type SSH struct {
 func NewSSH(ctx context.Context, router adapter.Router, logger log.ContextLogger, tag string, options option.SSHOutboundOptions) (*SSH, error) {
 	outbound := &SSH{
 		myOutboundAdapter: myOutboundAdapter{
-			protocol: C.TypeSSH,
-			network:  []string{N.NetworkTCP},
-			router:   router,
-			logger:   logger,
-			tag:      tag,
+			protocol:     C.TypeSSH,
+			network:      []string{N.NetworkTCP},
+			router:       router,
+			logger:       logger,
+			tag:          tag,
+			dependencies: withDialerDependency(options.DialerOptions),
 		},
 		ctx:               ctx,
 		dialer:            dialer.New(router, options.DialerOptions),
@@ -88,6 +95,15 @@ func NewSSH(ctx context.Context, router adapter.Router, logger log.ContextLogger
 		}
 		outbound.authMethod = append(outbound.authMethod, ssh.PublicKeys(signer))
 	}
+	if len(options.HostKey) > 0 {
+		for _, hostKey := range options.HostKey {
+			key, _, _, _, err := ssh.ParseAuthorizedKey([]byte(hostKey))
+			if err != nil {
+				return nil, E.New("parse host key ", key)
+			}
+			outbound.hostKey = append(outbound.hostKey, key)
+		}
+	}
 	return outbound, nil
 }
 
@@ -122,6 +138,18 @@ func (s *SSH) connect() (*ssh.Client, error) {
 		Auth:              s.authMethod,
 		ClientVersion:     s.clientVersion,
 		HostKeyAlgorithms: s.hostKeyAlgorithms,
+		HostKeyCallback: func(hostname string, remote net.Addr, key ssh.PublicKey) error {
+			if len(s.hostKey) == 0 {
+				return nil
+			}
+			serverKey := key.Marshal()
+			for _, hostKey := range s.hostKey {
+				if bytes.Equal(serverKey, hostKey.Marshal()) {
+					return nil
+				}
+			}
+			return E.New("host key mismatch, server send ", key.Type(), " ", base64.StdEncoding.EncodeToString(serverKey))
+		},
 	}
 	clientConn, chans, reqs, err := ssh.NewClientConn(conn, s.serverAddr.Addr.String(), config)
 	if err != nil {
@@ -146,6 +174,11 @@ func (s *SSH) connect() (*ssh.Client, error) {
 	return client, nil
 }
 
+func (s *SSH) InterfaceUpdated() error {
+	common.Close(s.clientConn)
+	return nil
+}
+
 func (s *SSH) Close() error {
 	return common.Close(s.clientConn)
 }
